from edc_constants.constants import CLINIC, HOME, HOSPITAL, NOT_APPLICABLE, TELEPHONE

list_data = {
    "edc_appointment.appointmenttype": [
        (CLINIC, "In clinic"),
        (HOME, "At home"),
        (HOSPITAL, "In hospital"),
        (TELEPHONE, "By telephone"),
        (NOT_APPLICABLE, "Not applicable"),
    ],
}
