# Generated by Django 2.0rc1 on 2017-11-28 19:53

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [("edc_appointment", "0011_auto_20171128_1054")]

    operations = [
        migrations.AlterModelOptions(
            name="appointment",
            options={"ordering": ("timepoint", "visit_code_sequence")},
        )
    ]
